import { defaultTocOption } from './toc-option';
const defaults = defaultTocOption;
const findHeadlineElements = (levels, tokens, options, md, env) => {
    const headings = [];
    let currentHeading = null;
    tokens.forEach(token => {
        if (token.type === 'heading_open') {
            const id = findExistingIdAttr(token);
            const level = parseInt(token.tag.toLowerCase().replace('h', ''), 10);
            if (levels.indexOf(level) >= 0) {
                currentHeading = {
                    level: level,
                    textContent: null,
                    anchor: id || null,
                    markdownContent: null,
                };
            }
        }
        else if (currentHeading && token.type === 'inline') {
            const textContent = token.children
                .filter(childToken => childToken.type === 'text' || childToken.type === 'code_inline')
                .reduce((acc, t) => acc + t.content, '');
            currentHeading.textContent = textContent;
            currentHeading.markdownContent = md.renderer.renderInline(token.children, options, env);
            if (!currentHeading.anchor) {
                currentHeading.anchor = options.slugify(textContent, token.content);
            }
        }
        else if (token.type === 'heading_close') {
            if (currentHeading) {
                headings.push(currentHeading);
            }
            currentHeading = null;
        }
    });
    return headings;
};
const findExistingIdAttr = (token) => {
    if (token && token.attrs && token.attrs.length > 0) {
        const idAttr = token.attrs.find(attr => {
            if (Array.isArray(attr) && attr.length >= 2) {
                return attr[0] === 'id';
            }
            return false;
        });
        if (idAttr && Array.isArray(idAttr) && idAttr.length >= 2) {
            const [, val] = idAttr;
            return val;
        }
    }
    return null;
};
function getMinLevel(headlineItems) {
    return Math.min(...headlineItems.map(item => item.level));
}
const addListItem = (level, text, anchor, rootNode) => {
    const listItem = {
        level,
        text,
        anchor,
        children: [],
        parent: rootNode,
    };
    rootNode.children.push(listItem);
    return listItem;
};
const flatHeadlineItemsToNestedTree = (headlineItems) => {
    const toc = {
        level: getMinLevel(headlineItems) - 1,
        anchor: null,
        text: null,
        children: [],
        parent: null,
    };
    let currentRootNode = toc;
    let prevListItem = currentRootNode;
    headlineItems.forEach(headlineItem => {
        if (headlineItem.level > prevListItem.level) {
            Array.from({ length: headlineItem.level - prevListItem.level }).forEach(_ => {
                currentRootNode = prevListItem;
                prevListItem = addListItem(headlineItem.level, null, null, currentRootNode);
            });
            prevListItem.text = headlineItem.markdownContent;
            prevListItem.anchor = headlineItem.anchor;
        }
        else if (headlineItem.level === prevListItem.level) {
            prevListItem = addListItem(headlineItem.level, headlineItem.markdownContent, headlineItem.anchor, currentRootNode);
        }
        else if (headlineItem.level < prevListItem.level) {
            for (let i = 0; i < prevListItem.level - headlineItem.level; i++) {
                currentRootNode = currentRootNode.parent;
            }
            prevListItem = addListItem(headlineItem.level, headlineItem.markdownContent, headlineItem.anchor, currentRootNode);
        }
    });
    return toc;
};
const tocItemToHtml = (tocItem, options, md) => {
    return ('<' +
        options.listType +
        '>' +
        tocItem.children
            .map(childItem => {
            let li = '<li>';
            let anchor = childItem.anchor;
            if (options && options.transformLink) {
                anchor = options.transformLink(anchor);
            }
            let text = childItem.text ? options.format(childItem.text, md, anchor) : null;
            li += anchor ? `<a href="#${anchor}">${text}</a>` : text || '';
            return li + (childItem.children.length > 0 ? tocItemToHtml(childItem, options, md) : '') + '</li>';
        })
            .join('') +
        '</' +
        options.listType +
        '>');
};
const plugin = (md, o) => {
    const options = Object.assign({}, defaults, o);
    const tocRegexp = options.markerPattern;
    let gState;
    const toc = (state, silent) => {
        let token;
        let match;
        if (state.src.charCodeAt(state.pos) !== 0x5b) {
            return false;
        }
        if (silent) {
            return false;
        }
        match = tocRegexp.exec(state.src.substr(state.pos));
        match = !match
            ? []
            : match.filter(function (m) {
                return m;
            });
        if (match.length < 1) {
            return false;
        }
        token = state.push('toc_open', 'toc', 1);
        token.markup = '[[toc]]';
        token = state.push('toc_body', '', 0);
        token = state.push('toc_close', 'toc', -1);
        const newline = state.src.indexOf('\n', state.pos);
        if (newline !== -1) {
            state.pos = newline;
        }
        else {
            state.pos = state.pos + state.posMax + 1;
        }
        return true;
    };
    md.renderer.rules.toc_open = (tokens, index) => {
        let tocOpenHtml = '<div class="' + options.containerClass + '">';
        if (options.containerHeaderHtml) {
            tocOpenHtml += options.containerHeaderHtml;
        }
        return tocOpenHtml;
    };
    md.renderer.rules.toc_close = (tokens, index) => {
        let tocFooterHtml = '';
        if (options.containerFooterHtml) {
            tocFooterHtml = options.containerFooterHtml;
        }
        return tocFooterHtml + '</div>';
    };
    md.renderer.rules.toc_body = (tokens, index, opt, env) => {
        if (options.forceFullToc) {
            throw 'forceFullToc was removed in version 0.5.0. For more information, see https://github.com/Oktavilla/markdown-it-table-of-contents/pull/41';
        }
        else {
            const headlineItems = findHeadlineElements(options.includeLevel, gState.tokens, options, md, env);
            const toc = flatHeadlineItemsToNestedTree(headlineItems);
            return tocItemToHtml(toc, options, md);
        }
    };
    md.core.ruler.push('grab_state', state => {
        gState = state;
        return true;
    });
    md.inline.ruler.after('emphasis', 'toc', toc);
};
export { plugin as TocPlugin };
//# sourceMappingURL=index.js.map